/******************************************************************************

    main.c

    Test demo program for multiple HID devices.

    Test hardware is SBDBT32 board, it has PIC32MX695F512H.
                     PIC24USB board, it has PIC24FJ256GB106 

    (C) S.Suwa 2013/11/22       http://www.suwa-koubou.jp

    2013/11/22 support multiple HID device.
               mouse, keyboard, joystick

******************************************************************************/

#include <stdio.h>
#include <string.h>
#include "GenericTypeDefs.h"
#include "HardwareProfile.h"
#include "usb.h"
#include "usb_host_hid.h"
#include "usb_host_hid_parser.h"
#include "usb_host_hid_device.h"
#include "uart2.h"


/*------------------------------------------------------------------------------
	Configuration for PIC24FJ256/PIC32MX695F 
------------------------------------------------------------------------------*/

#if defined(__PIC24FJ64GB002__)
    // PIC24FJ64GB002 
    _CONFIG1(WDTPS_PS1 & FWPSA_PR32 & WINDIS_OFF & FWDTEN_OFF & ICS_PGx1 & GWRP_OFF & GCP_OFF & JTAGEN_OFF)
    _CONFIG2(POSCMOD_NONE & I2C1SEL_PRI & IOL1WAY_OFF & OSCIOFNC_ON & FCKSM_CSDCMD & FNOSC_FRCPLL & PLL96MHZ_ON & PLLDIV_DIV2 & IESO_OFF)
    _CONFIG3(WPFP_WPFP0 & SOSCSEL_IO & WUTSEL_LEG & WPDIS_WPDIS & WPCFG_WPCFGDIS & WPEND_WPENDMEM)
    //_CONFIG3(WPFP_WPFP0 & SOSCSEL_SOSC & WUTSEL_LEG & WPDIS_WPDIS & WPCFG_WPCFGDIS & WPEND_WPENDMEM)
    _CONFIG4(DSWDTPS_DSWDTPS3 & DSWDTOSC_LPRC & RTCOSC_SOSC & DSBOREN_OFF & DSWDTEN_OFF)

#elif defined(__PIC24FJ256GB106__)
    // PIC24FJ256GB106 
    _CONFIG1( JTAGEN_OFF & GCP_OFF & GWRP_OFF & FWDTEN_OFF & ICS_PGx2 & BKBUG_OFF & COE_OFF ) 
    _CONFIG2( PLL_96MHZ_ON & IESO_OFF & FCKSM_CSDCMD & OSCIOFNC_OFF & POSCMOD_XT & FNOSC_PRIPLL & PLLDIV_DIV4 & IOL1WAY_OFF & DISUVREG_OFF) 
    _CONFIG3( WPEND_WPENDMEM & WPCFG_WPCFGDIS & WPDIS_WPDIS )

#elif defined(__32MX220F032B__)
	#include "p32mx220f_config.h"

#elif defined(__32MX695F512H__)
	#include "p32mx695f_config.h"
#else
	#error  "Undefined processor"
#endif


/*------------------------------------------------------------------------------
	definition
------------------------------------------------------------------------------*/

#define	putstr(x)	UART2PrintString(x)
#define	inkey()		UART2IsPressed()
#define	getch()		UART2GetChar()
#define	puthex(x)	UART2PutHex(x)
#define	putch(x)	UART2PutChar(x)
#define	putdec(x)	UART2PutDec(x)


/*------------------------------------------------------------------------------
	function prototype
------------------------------------------------------------------------------*/

void	InitializeSystem (void);
int     main (void);
void	mouse_test(void);
void    keyboard_test(void);
BYTE    toAscii(USB_KEYBOARD_DATA *data);
void	joystick_test(void);
void	getstr(char *buf, BYTE length);

/*------------------------------------------------------------------------------
	global variable
------------------------------------------------------------------------------*/


/*------------------------------------------------------------------------
	InitializeSystem()
-----------------------------------------------------------------------*/

void InitializeSystem ( void )
{
#if defined(__PIC24FJ64GB002__)
	unsigned int pll_startup_counter;

	// Fosc(system clock) 8MHz postscale = 1:1 x4 = 32MHz, Fcy = Fosc/2
	CLKDIV = 0;

	// PLL clock start up
	pll_startup_counter = 600;
	CLKDIVbits.PLLEN = 1;
	while(pll_startup_counter--);

	//--- setting I/O port ----------------------------------------
	AD1PCFG = 0xffff;	// Analog disable
	LATA = 0x0000; TRISA = 0xffff;	// RA are default input.
	LATB = 0x0000; TRISB = 0xffff;	// RB are default input.

	TRISAbits.TRISA0 = 0;   // RA0(UxTX) is output
	TRISAbits.TRISA1 = 1;   // RA1(UxRX) is input
	LATAbits.LATA0 = 1;     // RA0(UxTX) to initial level high

	//--- remapping I/O pin for peripheral ------------------------
	//unlock registers
	__builtin_write_OSCCONL(OSCCON & 0xBF);

	// Assign UART2	
	RPOR2bits.RP5R = 5;     // RA0/RP5(pin 2)->UART2 TX
	RPINR19bits.U2RXR = 6;	// RA1/RP6(pin 3)->UART2 RX

	//lock registers
	__builtin_write_OSCCONL(OSCCON | 0x40);

#elif defined(__PIC24FJ256GB106__)

	// Fosc(system clock) 8MHz postscale = 1:1 x4 = 32MHz, Fcy = Fosc/2
	CLKDIV = 0;


	//--- setting I/O port ----------------------------------------
	AD1PCFG = 0xffff;	// Analog disable
	LATB = 0x0000; TRISB = 0xffff;	// RB are default input.
	LATC = 0x0000; TRISC = 0xffff;	// RC are default input.
	LATD = 0x0000; TRISD = 0xffff;	// RD are default input.
	LATE = 0x0000; TRISE = 0xffff;	// RE are default input.
	LATF = 0x0000; TRISF = 0xffff;	// RF are default input.
	LATG = 0x0000; TRISG = 0xffff;	// RG are default input.

	//--- remapping I/O pin for peripheral ------------------------
	//unlock registers
	__builtin_write_OSCCONL(OSCCON & 0xBF);

	// Assign UART2	
	RPOR5bits.RP10R = 5;	// RF4/RP10 <- UART2 TX(5)
	RPINR19bits.U2RXR = 17;	// UART2RX  <- RF5/RP17

	TRISFbits.TRISF4 = 0;   // RF4(UxTX) is output
	TRISFbits.TRISF5 = 1;   // RF5(UxRX) is input
	LATFbits.LATF4 = 1;     // RF4(UxTX) to level high

	//lock registers
	__builtin_write_OSCCONL(OSCCON | 0x40);


#elif defined(__32MX220F032B__)
	int	value;

	// Set System Config
	//SYSTEMConfigPerformance(GetSystemClock());

	value = SYSTEMConfigWaitStatesAndPB( GetSystemClock() );

	// Enable the cache for the best performance	
	CheKseg0CacheOn();

	INTEnableSystemMultiVectoredInt();

	value = OSCCON;
	while (!(value & 0x00000020)){
		value = OSCCON;    // Wait for PLL lock to stabilize	
	}

	INTEnableInterrupts();

	// ADC input off
	ANSELA = 0;
	ANSELB = 0;

	// Assign UART2	
	U2RXRbits.U2RXR = 0;	// UART2 RX <- RPA1(pin 3)[0]
	RPA3Rbits.RPA3R = 2;	// RPA3(pin10)<- UART2 TX [2]

#elif defined(__32MX695F512H__)
	int	value;

	// Set System Config
	//SYSTEMConfigPerformance(GetSystemClock());

	value = SYSTEMConfigWaitStatesAndPB( GetSystemClock() );

	// Enable the cache for the best performance	
	CheKseg0CacheOn();

	INTEnableSystemMultiVectoredInt();

	value = OSCCON;
	while (!(value & 0x00000020)){
		value = OSCCON;    // Wait for PLL lock to stabilize	
	}

	INTEnableInterrupts();

	// ADC input off
	AD1PCFG = 0xffff;

	// set UART1 port
	TRISD = 0xffff;
	TRISDCLR = 0x0008; // to digital OUT for UART1 TX
	LATDSET = 0x0008; // default level high for UART1 TX

	// SBDBT32 set LED port 
	TRISB = 0xffff;
	TRISBCLR = 0xc000;	// RB14,RB15 are digital OUT.
	LATBSET = 0xc000;	// RB14(LED1), RB15(LED2) are turn off.

#else
	#error "Undefined processer"
#endif

    /*
     * Init UART
     *
     * caution:
     *  SBDBT32 must use UART1@because UART2 is not usable.
     *  I corrected 'uart2.c', in order to use UART1 instead of UART2 by USE_SBDBT32 macro switch. 
     *  Please see 'uart2.c' and 'uart2to1.c'.
     */
    UART2Init();

} // InitializeSystem


/*============================================================================================
	BOOL USB_ApplicationEventHandler ( BYTE address, USB_EVENT event, void *data, DWORD size )
============================================================================================*/

BOOL USB_ApplicationEventHandler ( BYTE address, USB_EVENT event, void *data, DWORD size )
{

    // Handle specific events.
    switch (event){

    case EVENT_OVERRIDE_CLIENT_DRIVER_SELECTION:
        return FALSE;
 
    case EVENT_VBUS_RELEASE_POWER:
    case EVENT_VBUS_REQUEST_POWER:
    case EVENT_UNSUPPORTED_DEVICE:
    case EVENT_CANNOT_ENUMERATE:
    case EVENT_CLIENT_INIT_ERROR:
    case EVENT_OUT_OF_MEMORY:
    case EVENT_UNSPECIFIED_ERROR:
        break;

    default:
       break;
   }

   return TRUE;

} // USB_ApplicationEventHandler


/*-------------------------------------------------------------------------------
	BOOL USB_ApplicationDataEventHandler ( BYTE address, USB_EVENT event, void *data, DWORD size )
-------------------------------------------------------------------------------*/

BOOL USB_ApplicationDataEventHandler ( BYTE address, USB_EVENT event, void *data, DWORD size )
{
	return TRUE;
}


/*==============================================================================
	main
===============================================================================*/

int main ( void )
{
    int  c;

    // Initialize the processor and peripherals.
    InitializeSystem();


    for(;;){
        // clear key buffer
        while(inkey()) getch();

        putstr("\r\n*** Demonstration program Menu *********************\r\n");
        putstr("m:Mouse read test.\r\n");
        putstr("k:Keyboard read test.\r\n");  
        putstr("j:Joystick read test.\r\n");
        putstr("=>");

        // wait for typed key. 
        while(!inkey()){
            USBTasks();
        }
        c = getch();

        if(c >= ' ' && c <= 0x7e){
            putch(c);
            putstr("\r\n");
        }

        switch(c){
            // mouse read test.		
            case 'm':
                if(getHIDnumOfDevice(HID_MOUSE)==0){
                    putstr("Mouse not connected. Can't test!\r\n");
                }else{
                    mouse_test();
                }
                break;

            // usb keyboard read test.
            case 'k':
                if(getHIDnumOfDevice(HID_KEYBOARD)==0){
                    putstr("Keyboard not connected. Can't test!\r\n");
                }else{
                    keyboard_test();
                }
                break;

            // usb joystick read test.
            case 'j':
               if(getHIDnumOfDevice(HID_JOYSTICK)==0){
                   putstr("Joystick not connected. Can't test!\r\n");
               }else{
                   joystick_test();
               }
               break;

            default:
               break;

            } // switch(c)

    } // for (menu loop)

    return 0;
}



/*===============================================================================
	mouse test
===============================================================================*/

void	mouse_test(void)
{
	void displayMouse(USB_MOUSE_DATA *moused);

	USB_MOUSE_DATA   moused;
	int rc, n, numMouse;

	putstr("Start mouse test. \r\n");
	putstr(" Push any keys of PC keyboards when you will stop the mouse test.\r\n"); 

	for(;;){

		if(inkey()){
			putstr("\r\nYou pressed any keys, to stop the mouse test! \r\n");
			return;
		}

		if((numMouse=getHIDnumOfDevice(HID_MOUSE)) == 0){
		    putstr("\r\nMouse detached, to stop the mouse test! \r\n");
		    return;
		}

		for(n = 0; n < numMouse; n++){
			rc = mouse_read(n+1, &moused);
			if(rc == 1){
				displayMouse(&moused);
			}
		}
	}
}

void displayMouse(USB_MOUSE_DATA *moused)
{
	if(moused->button.left){
		putstr(" left button on.\r\n");
	}

	if(moused->button.right){
		putstr(" right button on. \r\n");
	}

	if(moused->button.wheel){
		putstr(" wheel button on. \r\n");
	}

	if(moused->x_axis > 0){
		if( moused->y_axis > 0){
			putstr(" move to right down.\r\n");
		}

		if( moused->y_axis == 0){
			putstr(" move to right.\r\n");
		}

		if( moused->y_axis < 0){
			putstr(" move to right up.\r\n");
		}
	}

	if(moused->x_axis == 0){
		if( moused->y_axis > 0){
			putstr(" move to down.\r\n");
		}
		if( moused->y_axis < 0){
			putstr(" move to up.\r\n");
		}
	}
		
	if(moused->x_axis < 0){
		if( moused->y_axis > 0){
			putstr(" move to left down.\r\n");
		}

		if( moused->y_axis == 0){
			putstr(" move to left.\r\n");
		}

		if( moused->y_axis < 0){
			putstr(" move to left up.\r\n");
		}
	}

	if(moused->wheel < 0){
		putstr(" wheel up.\r\n");
	}

	if(moused->wheel > 0){
		putstr(" wheel down.\r\n");
	}
}


/*===============================================================================
	keyboard test (added this test function, 2013/11/13)
===============================================================================*/

void	keyboard_test(void)
{
	USB_KEYBOARD_DATA   keyd;
	int rc;
	BYTE c, prev;


	putstr("\r\n*** Start USB keyboard test. \r\n");
	putstr("Please press a key of USB keyboard.\r\n");
	putstr(" If you press Ctrl+C, to stop the keyboard test.\r\n"); 

	for(c = prev = 0;;){

		do {
			rc = keyboard_read(1, &keyd);
			if(rc == -1){
				putstr("\r\nKeyboard detached. Stop the keyboard test! \r\n");
				return;
			}
		}while(rc == 0);

		if((keyd.modifier.left_ctrl || keyd.modifier.right_ctrl) &&
			keyd.keycode[0] == 0x06){
				putstr("\r\nYou pressed Ctrl+C, to stop the keyborad test! \r\n");
				return;
		}

		c = toAscii(&keyd);
		if(c && c != prev) putch(c);
		if(c == '\r') putch('\n');
		prev = c;

	}

}


/*------------------------------------------------------------------------------
	BYTE toAscii()
------------------------------------------------------------------------------*/

const BYTE tblAscii[]={
/*           00  01  02  03  04  05  06  07  08  09  0A  0B  0C  0D  0E  0F */
/*00-0f*/     0,  0,  0,  0,'a','b','c','d','e','f','g','h','i','j','k','l',
/*10-1f*/   'm','n','o','p','q','r','s','t','u','v','w','x','y','z','1','2',
/*20-2f*/   '3','4','5','6','7','8','9','0','\r', 0,0x08,'\t',' ','-','^','@',
/*30-3f*/   '[',  0,']',';',':', 0 ,',','.','/',  0,  0,  0,  0,  0,  0,  0,
/*40-4f*/     0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
/*50-5f*/     0,  0,  0,  0,'/','*','-','+',  0,'1','2','3','4','5','6','7',
/*60-6f*/   '8','9','0',  0,  0,  0,  0,'\\',  0,'\\',  0,  0,  0,  0,  0,  0,
};

const BYTE tblAsciiWithShift[]={
/*           00  01  02  03  04  05  06  07  08  09  0A  0B  0C  0D  0E  0F */
/*00-0f*/     0,  0,  0,  0,'A','B','C','D','E','F','G','H','I','J','K','L',
/*10-1f*/   'M','N','O','P','Q','R','S','T','U','V','W','X','Y','Z','!','"',
/*20-2f*/   '#','$','%','&','\'','(',')','0','\r', 0,0x08,'\t',' ','=','~','`',
/*30-3f*/   '{',  0,'}','+','*',  0,'<','>','?',  0,  0,  0,  0,  0,  0,  0,
/*40-4f*/     0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
/*50-5f*/     0,  0,  0,  0,'/','*','-','+',  0,'1','2','3','4','5','6','7',
/*60-6f*/   '8','9','0',  0,  0,  0,  0,'_',  0,'|',  0,  0,  0,  0,  0,  0,
};
 
BYTE toAscii(USB_KEYBOARD_DATA *data)
{
	if(data->keycode[0] == 0x87 || data->keycode[0] == 0x89)data->keycode[0] -= 0x20;

	if(data->keycode[0] >= 0x70) return 0;

	if(data->modifier.left_shift || data->modifier.right_shift){
		return tblAsciiWithShift[data->keycode[0]];

	}else if(data->keycode[0] <= 0x1d && data->modifier.capslock){
		return tblAsciiWithShift[data->keycode[0]];

	}else{
		return tblAscii[data->keycode[0]];
	}
}


/*===============================================================================
	joystick test  (added this test function, 2013/11/13)
===============================================================================*/

void	joystick_test(void)
{
	USB_JOYSTICK_DATA   joyd;
	int rc, n;
	WORD btn, prev_btn;
	short prev_x, prev_y, prev_z, prev_s, prev_h;

	putstr("Start joystick test. \r\n");
	putstr(" Push any keys of PC keyboard, when you will stop the joystick test.\r\n"); 

	prev_x = prev_y = prev_z = prev_s = prev_h = prev_btn = 0;
	for(;;){

		do {
			rc = joystick_read(1, &joyd);
			if(rc == -1 || inkey()){
				putstr("\r\nJoystick detached or You pressed any keys, to stop the joystick test! \r\n");
				return;
			}
		}while(rc == 0);

		if(joyd.x_axis != prev_x){
			putstr("X="); putdec(joyd.x_axis); putstr("\r\n");
			prev_x = joyd.x_axis;
		}

		if(joyd.y_axis != prev_y){
			putstr("Y="); putdec(joyd.y_axis); putstr("\r\n");
			prev_y = joyd.y_axis;
		}

		if(joyd.z_axis != prev_z){
			putstr("Z="); putdec(joyd.z_axis); putstr("\r\n");
			prev_z = joyd.z_axis;
		}

		if(joyd.rz_axis != prev_s){
			putstr("R="); putdec(joyd.rz_axis); putstr("\r\n");
			prev_s = joyd.rz_axis;
		}

		if(joyd.hatswitch != prev_h){
			putstr("HatSwitch="); putdec(joyd.hatswitch); putstr("\r\n");
			prev_h = joyd.hatswitch;
		}

		btn = *((WORD *)&(joyd.button));
		if(!btn){                   // fix. 2013/11/22
			prev_btn = 0;
			continue;
		}
		if(btn == prev_btn) continue;

		prev_btn = btn;
		putstr("btn: 1  2  3  4  5  6  7  8  9 10 11 12\r\n");
		putstr("    ");
		for(n = 0; n < 12; n++){
			if(btn & 1){
				putstr("on ");
			}else{
				putstr("   ");
			}
			btn >>= 1;
		}
		putstr("\r\n");
	}
}


/**** end of main.c ************************************************************/
